/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#ifndef __idt_h__
#define __idt_h__

#define IDT_CS_SELECTOR 0x08 // Selector where runs the handler.
#define IDT_BASE 0x6800 // Base IDT address
#define IDT_BASE_L 0x6800
#define IDT_BASE_H 0

// Type of gate
#define IDT_CALL_GATE 0x4 // Call gate
#define IDT_TASK_GATE 0x5 // Task gate
#define IDT_INT_GATE 0x6 // Interrupt gate
#define IDT_TRAP_GATE 0x7 // Trap gate
// Size
#define IDT_SIZE_16 0x0 // Only for 286 or lower
#define IDT_SIZE_32 0x08 // Default
// Privilege level
#define IDT_RING_0 0x0 // More privilege (kernel)
#define IDT_RING_1 0x2
#define IDT_RING_2 0x4
#define IDT_RING_3 0x6 // Less privilege (user)
// Present
#define IDT_NOT_PRESENT 0x00 // Undefined rescriptor
#define IDT_PRESENT 0x80 // Valid descriptor

// IDT descriptor entry
typedef struct idt_entry_struct
{
	unsigned short int offset_l; // Low Offset handler (0-15)	
	unsigned short int selector; // GDT selector where handler runs
	unsigned char unused;		
	unsigned char options; // Descriptor options
	unsigned short int offset_h; // High Offset handler (16-31)
}tp_idt_entry;

// IDT register
typedef struct idt_register_struct
{
	unsigned short int limit; // IDT limit
	unsigned short int base_l; // IDT base address low (0-15)
	unsigned short int base_h; // IDT base address high (16-31)
}tp_idtr;

void init_idt();
void load_idt_entry(int idt_num, unsigned int offset,
	unsigned int selector, unsigned char options);
void load_idtr();

#endif
