/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#include <console.h>
#include <io.h>
#include <pic.h>

static char irq_status_master = 0xFF;
static char irq_status_slave = 0xFF;
 
void init_pic()
{
	// Send ICW1
	outb(PIC1, ICW1);
	outb(PIC2, ICW1);
	// Send ICW2
	outb(PIC1 + 1, ICW2_1);	
	outb(PIC2 + 1, ICW2_2);	
	// Send ICW3
	outb(PIC1 + 1, ICW3_1);	
	outb(PIC2 + 1, ICW3_2);
	// Send ICW4
	outb(PIC1 + 1, ICW4);
	outb(PIC2 + 1, ICW4);

	// Disable all IRQ's
	outb(PIC1 + 1, 0xFF); // Set all bits of PIC1
	outb(PIC2 + 1, 0xFF); // Set all bits of PIC2
	
	put_string("PIC Initialization [ DONE ]\n");
}

void enable_irq(unsigned char irq_num)
{
	/* IRQ's 0,1,3..7 PIC MASTER (use IRQ2 to enable PIC SLAVE)
	 * IRQ's 8..15 PIC SLAVE (IRQ9==IRQ2) */
	
	unsigned char mask;
	mask = ~(1 << (irq_num & 7)); // (irq_num & 7) => irq_num mod(8) 
																// All bits set except irq_num mod(8) bit
	
	if (irq_num < 8)	{ 
		// Master PIC
		irq_status_master &= mask; // Deactivate irq_num mod(8) bit in status
		outb(PIC1+1, irq_status_master); // Change th status of PIC1
	}else{
		// Slave PIC
		irq_status_slave &= mask; // Deactivate irq_num mod(8) bit in status
		outb(PIC2+1, irq_status_slave); // Change th status of PIC2
	}	
	
}
